package com.avif.demo

import android.graphics.Bitmap
import android.graphics.BitmapFactory
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.util.Log
import android.widget.Button
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.avif.demo.adapter.MyAdapter
import com.avif.demo.dialog.LoadingDialog
import com.avif.demo.entity.CsvEntity
import com.avif.demo.entity.EncodeResult
import com.avif.demo.util.CsvUtil
import com.avif.demo.util.EncodeUtil
import com.avif.demo.util.GlideUtil
import com.avif.demo.util.LogUtil

class MainActivity : AppCompatActivity() {

    private val TAG = "MainActivity"

    private lateinit var mCsvEntities: List<CsvEntity>
    private var mEncodeResults: MutableList<EncodeResult?> = ArrayList()
    private var mAdapter: MyAdapter? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)
        initCsv()

        findViewById<Button>(R.id.button).setOnClickListener {
            // 验证并展示csv测试集时，请调用star()
            // 如果要使用assets中的图片进行编码测试时，可以调用encodeTest()
            star()
//            encodeTest()
        }

        val recyclerView = findViewById<RecyclerView>(R.id.recyclerView)
        recyclerView.layoutManager = LinearLayoutManager(this)

        mAdapter = MyAdapter(this)
        recyclerView.adapter = mAdapter
    }

    /**
     * 用assets中的文件测试编码
     */
    private fun encodeTest() {
        var inputStream = assets.open("512_big.jpg")
        var bitmap = BitmapFactory.decodeStream(inputStream)
        Log.e(TAG, "================>${bitmap.width}x${bitmap.height}")
        EncodeUtil.encodeRGBToAvif(this@MainActivity, bitmap)
    }

    private fun initCsv() {
        mCsvEntities = CsvUtil.parseCsv(this)
    }

    /**
     * 开始下载csv中的图片文件并编码及展示
     */
    private fun star() {
        mEncodeResults = ArrayList()
        showLoading()
        download()
    }

    private fun download() {
        mCsvEntities.forEach {
            GlideUtil.downloadBitmap(this, it.url, object : BitmapCallback {
                override fun onSuccess(bitmap: Bitmap) {
                    Log.e(TAG, "======================= ${it.obj} =======================")
                    Log.e(TAG, "=====>图片尺寸:${bitmap.width}x${bitmap.height}")
                    encode(bitmap)
                }

                override fun onError(e: Exception) {
                    Log.e(TAG, "$e")
                }
            })
        }
    }

    private fun encode(bitmap: Bitmap) {
        try {
            val result = EncodeUtil.encodeRGBToAvif(this@MainActivity, bitmap)
            mEncodeResults.add(result)
            LogUtil.encodeLog(result)
        } catch (e: Exception) {
            mEncodeResults.add(EncodeResult(null, 0, bitmap.width, bitmap.height))
        }
        runOnUiThread{
            if (mEncodeResults.size < mCsvEntities.size) {
                setLoadingTest("${mEncodeResults.size}/${mCsvEntities.size}")
            } else {
                dismissLoading()
                refreshList()
            }
        }
    }

    private fun refreshList() {
        mAdapter?.setData(mCsvEntities, mEncodeResults)
        mAdapter?.notifyDataSetChanged()
    }

    private fun showLoading() {
        LoadingDialog.getInstance(this).show()
    }

    private fun dismissLoading() {
        LoadingDialog.getInstance(this).dismiss()
    }

    private fun setLoadingTest(str: String) {
        LoadingDialog.getInstance(this).setText(str)
    }

}