package com.avif.demo.adapter

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.avif.demo.R
import com.avif.demo.entity.CsvEntity
import com.avif.demo.entity.EncodeResult
import com.avif.demo.util.DecoderUtil
import com.bumptech.glide.Glide

class MyAdapter(private val mContext: Context) : RecyclerView.Adapter<MyAdapter.ViewHolder>() {

    private var mCsvData: List<CsvEntity>? = null
    private var mEncodeData: List<EncodeResult?>? = null

    fun setData(csvData: List<CsvEntity>, encodeData: List<EncodeResult?>) {
        mCsvData = csvData
        mEncodeData = encodeData
        notifyDataSetChanged()
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val itemView = LayoutInflater.from(mContext).inflate(R.layout.layout_item, parent, false)
        return ViewHolder(itemView)
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        val data = mEncodeData?.get(position)
        Glide.with(mContext).asBitmap().load(mCsvData?.get(position)?.url).into(holder.originalImage)

        if (data != null) {
            if (data?.file != null) {
                val bmp = DecoderUtil.decodeAvif(data?.file!!)
                if (bmp != null) {
                    holder.encodeImage.setImageBitmap(bmp)
                }
                holder.timeTv.text = "编码后，编码耗时：${data.differTime}ms"
            }
            holder.sizeTv.text = "原图：${data.width}x${data.height}"
        }

    }

    override fun getItemCount(): Int {
        return mCsvData?.size ?: 0
    }

    inner class ViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {

        var originalImage: ImageView
        var encodeImage: ImageView
        var sizeTv: TextView
        var timeTv: TextView

        init {
            originalImage = itemView.findViewById(R.id.imageOriginalView)
            encodeImage = itemView.findViewById(R.id.imageEncodeView)
            sizeTv = itemView.findViewById(R.id.imageSizeText)
            timeTv = itemView.findViewById(R.id.encodeTimeText)
        }

    }

}