package com.avif.demo.util

import android.graphics.Bitmap
import android.os.Build
import android.util.Log
import org.aomedia.avif.android.AvifDecoder
import java.io.File
import java.io.FileInputStream
import java.io.InputStream
import java.nio.ByteBuffer

object DecoderUtil {

    private const val TAG = "DecoderUtil"

    /**
     * 通过File解码Avif图片
     */
    fun decodeAvif(file: File): Bitmap? {
        val startTime = System.currentTimeMillis()
        var stream: InputStream?
        try {
            stream = FileInputStream(file.absolutePath)
            val sourceCopy = maybeCopyBuffer(stream)
            val info = AvifDecoder.Info()
            if (!AvifDecoder.getInfo(sourceCopy, sourceCopy.remaining(), info)) {
                if (Log.isLoggable(TAG, Log.ERROR)) {
                    Log.e(TAG, "Failed to retrieve image information through ByteBuffer.")
                }
            }
            var bitmap: Bitmap? = null
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                bitmap = Bitmap.createBitmap(
                    info.width,
                    info.height,
                    Bitmap.Config.ARGB_8888
                )
            }
            if (!AvifDecoder.decode(sourceCopy, sourceCopy.remaining(), bitmap, 1)) {
                if (Log.isLoggable(TAG, Log.ERROR)) {
                    Log.e(TAG, "Failed to decode ByteBuffer as Avif.")
                }
            }
            val endTime = System.currentTimeMillis()
            Log.e(TAG, "======================= ${file.name} =======================")
            Log.e(TAG, "=====>解码后图片尺寸:${info.width}x${info.height}")
            Log.e(TAG, "=====>解码耗时:${endTime - startTime}ms")
            return bitmap
        } catch (e: Exception) {
            Log.e("BitmapFactory", "Unable to decode stream: $e")
        }
        return null
    }

    /**
     * InputStream转ByteBuffer
     */
    private fun maybeCopyBuffer(b: InputStream): ByteBuffer {
        val c = ByteArrayUtil.inputStreamToByteArray(b)
        val source = ByteBuffer.wrap(c)
        if (source.isDirect) {
            return source
        }
        val sourceCopy = ByteBuffer.allocateDirect(source.remaining())
        sourceCopy.put(source)
        sourceCopy.flip()
        return sourceCopy
    }

}