package com.avif.demo.util

import android.content.Context
import android.graphics.Bitmap
import com.avif.demo.entity.EncodeResult
import org.aomedia.avif.android.AvifDecoder
import java.io.*
import java.nio.ByteBuffer
import java.text.SimpleDateFormat
import java.util.*

object EncodeUtil {

    private const val TAG = "EncodeUtil"

    /**
     * RGB编码
     */
    fun encodeRGBToAvif(context: Context, bmp: Bitmap): EncodeResult {
        val startTime = System.currentTimeMillis()
        var pixelBuffer = ByteBuffer.allocateDirect(bmp.width * bmp.height * 4)
        bmp.copyPixelsToBuffer(pixelBuffer)
        var avifBytes = AvifDecoder.encodeRGBA8888(
            pixelBuffer,
            bmp.width * bmp.height * 4,
            bmp.width,
            bmp.height
        )

        val endTime = System.currentTimeMillis()

        var avifFile = createFile(context, "avif")
        var fos = FileOutputStream(avifFile)
        fos.use {
            it.write(avifBytes)
            it.flush()
        }

        return EncodeResult(avifFile, endTime - startTime, bmp.width, bmp.height)
    }

    /**
     * 创建图片文件
     */
    private fun createFile(context: Context, extension: String): File {
        val sdf = SimpleDateFormat("yyyy_MM_dd_HH_mm_ss_SSS", Locale.US)
        return File(context.getExternalFilesDir("avif"), "IMG_${sdf.format(Date())}.$extension")
    }

}